@@ -1,18 +1,18 @@
 ;;;-*- Mode: Lisp; Package: (X8664 :use CL) -*-
 ;;;
-;;;   Copyright (C) 2005-2009 Clozure Associates and contributors.
-;;;   This file is part of Clozure CL.  
+;;; Copyright 2005-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (defpackage "X8664"
   (:use "CL")
@@ -194,6 +194,7 @@
 (defx86reg arg_x.w r8w)
 (defx86reg arg_x.b r8b)
 
+
 (defx86reg temp1 r9)
 (defx86reg temp1.l r9d)
 (defx86reg temp1.w r9w)
@@ -212,31 +213,36 @@
 #+(or darwin-target windows-target)
 (defx86reg rcontext r11)
 
-(defx86reg save3 r11)
-(defx86reg save3.l r11d)
-(defx86reg save3.w r11w)
-(defx86reg save3.b r11b)
+(defx86reg temp6 r11)
+(defx86reg temp6.l r11d)
+(defx86reg temp6.w r11w)
+(defx86reg temp6.b r11b)
 
+(defx86reg arg_w r12)
+(defx86reg arg_w.l r12d)
+(defx86reg arg_w.w r12w)
+(defx86reg arg_w.b r12b)
+
+(defx86reg temp5 r12)
+(defx86reg temp5.l r12d)
+(defx86reg temp5.w r12w)
+(defx86reg temp5.b r12b)
 
-(defx86reg save2 r12)
-(defx86reg save2.l r12d)
-(defx86reg save2.w r12w)
-(defx86reg save2.b r12b)
 
 (defx86reg fn r13)
 (defx86reg fn.l r13d)
 (defx86reg fn.w r13w)
 (defx86reg fn.b r13b)
 
-(defx86reg save1 r14)
-(defx86reg save1.l r14d)
-(defx86reg save1.w r14w)
-(defx86reg save1.b r14b)
+(defx86reg temp4 r14)
+(defx86reg temp4.l r14d)
+(defx86reg temp4.w r14w)
+(defx86reg temp4.b r14b)
 
-(defx86reg save0 r15)
-(defx86reg save0.l r15d)
-(defx86reg save0.w r15w)
-(defx86reg save0.b r15b)
+(defx86reg temp3 r15)
+(defx86reg temp3.l r15d)
+(defx86reg temp3.w r15w)
+(defx86reg temp3.b r15b)
 
 ;;; Use xmm regs for floating-point.  (They can also hold integer values.)
 (defx86reg fp0 xmm0)
@@ -431,7 +437,10 @@
 (defconstant misc-dfloat-offset misc-data-offset)
 (defconstant misc-symbol-offset (- node-size fulltag-symbol))
 (defconstant misc-function-offset (- node-size fulltag-function))
-  
+;;; There is a pad word after the uvector header so that the
+;;; complex-double-float elements are 16-byte aligned.
+(defconstant misc-complex-dfloat-offset (+ misc-data-offset node-size))
+
 (define-subtag single-float fulltag-imm-0 0)
 
 (define-subtag character fulltag-imm-1 0)
@@ -550,10 +559,6 @@
   rbp
   foreign-sp
   db-link                               ; value of dynamic-binding link on thread entry.
-  save-save3                            ; saved nvrs
-  save-save2
-  save-save1
-  save-save0
   xframe                                ; exception-frame link
   pc                                    ; tra of catch exit/unwind cleanup
 )
@@ -868,6 +873,8 @@
   lisp-gettimeofday
   lisp-sigexit
   jvm-init
+  lisp-lstat
+  lisp-realpath
 )
 
 (defmacro nrs-offset (name)
@@ -1381,9 +1388,9 @@
 
 (defconstant arg-check-trap-pc-limit 7)
 
-(defconstant fasl-version #x60)
-(defconstant fasl-max-version #x60)
-(defconstant fasl-min-version #x60)
-(defparameter *image-abi-version* 1039)
+(defconstant fasl-version #x62)
+(defconstant fasl-max-version #x62)
+(defconstant fasl-min-version #x62)
+(defparameter *image-abi-version* 1042)
 
 (provide "X8664-ARCH")
