@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (in-package "CCL")
 
@@ -832,14 +831,11 @@
         (make-acode (%nx1-operator or) (nx1-formlist context (%cdr whole)))))))
 
 (defun nx1-1d-vref (context env arr dim0 &optional uvref-p)
+  (declare (ignorable context))
   (let* ((simple-vector-p (nx-form-typep arr 'simple-vector env))
-         (string-p (unless simple-vector-p 
-                     (if (nx-form-typep arr 'string env)
-                       (or (nx-form-typep arr 'simple-string env)
-                           (return-from nx1-1d-vref (nx1-form context `(char ,arr ,dim0)))))))
-         (simple-1d-array-p (unless (or simple-vector-p string-p) 
+	 (simple-string-p (unless simple-vector-p (nx-form-typep arr 'simple-string env)))
+         (simple-1d-array-p (unless (or simple-vector-p simple-string-p)
                               (nx-form-typep arr '(simple-array * (*)) env)))
-         
          (array-type (specifier-type  (nx-form-type arr env)))
          (type-keyword (funcall
                         (arch::target-array-type-name-from-ctype-function
@@ -851,7 +847,7 @@
                   (nx1-form :value arr)
                   (nx1-form :value dim0))
       (let* ((op (cond (simple-1d-array-p (%nx1-operator uvref))
-                       (string-p (%nx1-operator %sbchar))
+                       (simple-string-p (%nx1-operator %sbchar))
                        (simple-vector-p 
                         (if (nx-inhibit-safety-checking env) (%nx1-operator %svref) (%nx1-operator svref)))
                        (uvref-p (%nx1-operator uvref))
@@ -1158,7 +1154,6 @@
   ;; Validate the "read-only-p" argument
   (if (and read-only-p (neq read-only-p t)) (require-type read-only-p '(member t nil)))
   ;; Then ignore it.
-  (if *nx-load-time-eval-token*
     (multiple-value-bind (function warnings)
                          (compile-named-function 
                           `(lambda () ,form)
@@ -1167,8 +1162,12 @@
                           :load-time-eval-token *nx-load-time-eval-token*
                           :target (backend-name *target-backend*))
       (setq *nx-warnings* (append *nx-warnings* warnings))
-      (nx1-immediate context (list *nx-load-time-eval-token* `(funcall ,function))))
-    (nx1-immediate context (eval form))))
+      (if *nx-load-time-eval-token*
+        (nx1-immediate context (list *nx-load-time-eval-token* `(funcall ,function)))
+    
+      (make-acode (%nx1-operator load-time-value)
+              (make-acode (%nx1-operator immediate)
+                          (funcall function))))))
 
 (defun nx1-catch-body (context body)
   (let* ((temp (new-lexical-environment *nx-lexical-environment*)))
@@ -1229,13 +1228,15 @@
 
 
 (defnx1 nx1-%defun %defun context (&whole w def &optional (doc nil doc-p) &environment env)
-  (declare (ignorable doc doc-p))
+  (declare (ignorable doc doc-p def))
+  (when *backend-use-linear-scan*
+    (linear-scan-bailout '%defun))
   ;; Pretty bogus.
   (if (and (consp def)
            (eq (%car def) 'nfunction)
            (consp (%cdr def))
            (or (symbolp (%cadr def)) (setf-function-name-p (%cadr def))))
-    (note-function-info (%cadr def) (caddr def) env))
+    (note-function-info (%cadr def) (caddr def) env)) 
   (nx1-treat-as-call context w))
 
 (defnx1 nx1-function function context (arg &aux fn afunc)
@@ -1934,6 +1935,7 @@
     (multiple-value-bind (ok req opttail resttail keytail) (verify-lambda-list lambda-list)
       (declare (ignore req opttail))
       (when (and ok (or (eq (%car resttail) '&lexpr)
+                        ;*backend-use-linear-scan*
                         (eq (%car keytail) '&key)))
         (return-from nx1-lambda-bind (nx1-call context (nx1-form context `(lambda ,lambda-list ,@body)) args))))
     (let* ((*nx-lexical-environment* body-environment)
