@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 
 ;;; This file contains the error/condition system.  Functions that
@@ -55,7 +54,7 @@
 
 (define-condition storage-condition (serious-condition) ())
 
-(define-condition thread-condition (serious-condition) ())
+(define-condition thread-condition (condition) ())
 
 (define-condition process-reset (thread-condition)
   ((kill :initarg :kill :initform nil :reader process-reset-kill)))
@@ -424,6 +423,22 @@
   (:report (lambda (c s)
              (format s (%rsc-string $xunintc) (slot-value c 'sym-to-unintern) (package-error-package c) (slot-value c 'conflicting-syms)))))
 
+(define-condition simple-package-error (package-error simple-condition) ())
+
+(defun signal-package-error (package format-control &rest format-args)
+  (error 'simple-package-error
+         :package package
+         :format-control format-control
+         :format-arguments format-args))
+
+(defun signal-package-cerror (package continue-string
+                              format-control &rest format-args)
+  (cerror continue-string
+          'simple-package-error
+          :package package
+          :format-control format-control
+          :format-arguments format-args))
+
 (define-condition import-conflict-error (package-error)
   ((imported-sym :initarg :imported-sym)
    (conflicting-sym :initarg :conflicting-sym)
@@ -760,6 +775,12 @@
 	    (t				; with-simple-restart
 	     (throw tag (values nil t)))))))
 
+(define-condition restart-failure (control-error)
+  ((restart :initarg :restart :reader restart-failure-restart))
+  (:report (lambda (c s)
+             (format s "The ~a restart failed to transfer control dynamically as expected."
+                     (restart-name (restart-failure-restart c))))))
+
 (defun invoke-restart-no-return (restart &rest values)
   (declare (dynamic-extent values))
   (apply #'invoke-restart restart values)
