@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 ;;; backtrace.lisp
 ;;; low-level support for stack-backtrace printing
@@ -28,6 +27,7 @@
 (defparameter *backtrace-show-internal-frames* nil)
 (defparameter *backtrace-print-level* 2)
 (defparameter *backtrace-print-length* 5)
+(defparameter *backtrace-print-string-length* :default)
 
 (defparameter *backtrace-format* :traditional
   "If :TRADITIONAL, shows calls to non-toplevel functions using FUNCALL, and shows frame address values.
@@ -57,6 +57,7 @@
                           (start-frame-number 0)
                           (print-level *backtrace-print-level*)
                           (print-length *backtrace-print-length*)
+                          (print-string-length *backtrace-print-string-length*)
                           (show-internal-frames *backtrace-show-internal-frames*))
   "Returns a list representing the backtrace.
 Each element in the list is a list that describes the call in one stack frame:
@@ -65,6 +66,7 @@ The arguments are represented by strings, the function is a symbol or a function
 object."
   (let* ((*backtrace-print-level* print-level)
          (*backtrace-print-length* print-length)
+         (*backtrace-print-string-length* print-string-length)
          (*backtrace-format* :list)
          (result nil))
     (map-call-frames (lambda (p context)
@@ -90,10 +92,12 @@ object."
                                 (stream *debug-io*)
                                 (print-level *backtrace-print-level*)
                                 (print-length *backtrace-print-length*)
+                                (print-string-length *backtrace-print-string-length*)
                                 (show-internal-frames *backtrace-show-internal-frames*)
                                 (format *backtrace-format*))
   (let ((*backtrace-print-level* print-level)
         (*backtrace-print-length* print-length)
+        (*backtrace-print-string-length* print-string-length)
         (*backtrace-format* format)
         (*standard-output* stream)
         (*print-circle* nil)
@@ -154,7 +158,8 @@ object."
 	    (format t "~&  ~D " i)
 	    (when name (format t "~s" name))
 	    (let* ((*print-length* (default-print-length *backtrace-print-length*))
-		   (*print-level* (default-print-level *backtrace-print-level*)))
+		   (*print-level* (default-print-level *backtrace-print-level*))
+                   (*print-string-length* (default-print-string-length *backtrace-print-string-length*)))
 	      (format t ": ~s" var))
 	    (when type (format t " (~S)" type)))))
     (error () (format t "#<error printing frame>")))
@@ -170,7 +175,8 @@ object."
                (format t "~&     Arguments: ~:s" (arglist-from-map lfun)))
             (t (format t "~&  ~s" (arglist-from-map lfun))))
 	  (let* ((*print-length* (default-print-length *backtrace-print-length*))
-		 (*print-level* (default-print-level *backtrace-print-level*)))
+		 (*print-level* (default-print-level *backtrace-print-level*))
+                 (*print-string-length* (default-print-string-length *backtrace-print-string-length*)))
 	    (flet ((show-pair (pair prefix)
 		     (destructuring-bind (name . val) pair
 		       (format t "~&~a~s: " prefix name)
@@ -220,7 +226,8 @@ object."
             collect (if (eq arg (%unbound-marker))
                       "?"
                       (let* ((*print-length* (default-print-length *backtrace-print-length*))
-                             (*print-level* (default-print-level *backtrace-print-level*)))
+                             (*print-level* (default-print-level *backtrace-print-level*))
+                             (*print-string-length* (default-print-string-length *backtrace-print-string-length*)))
                         (format nil "~s" arg)))))))
 
 ;;; Return a list of "interesting" frame addresses in context, most
@@ -640,6 +647,7 @@ object."
                        (dolist (r rest) (push r result)))
                      (return))
                     (&rest
+                     (unless (listp value) (setf value (list value)))
                      (dolist (r value) (push r result))
                      (return))
                     (&key (push param result)))
