@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (in-package "CCL")
 
@@ -26,6 +25,7 @@
 (defparameter *trace-max-indent* 40)
 (defvar *trace-print-level* :default)
 (defvar *trace-print-length* :default)
+(defvar *trace-print-string-length* :default)
 ;(defparameter *trace-define-if-undefined* nil)
 (defparameter *trace-bar-frequency* nil)
 (defvar *trace-hook* nil)
@@ -87,6 +87,7 @@
   (trace-tab :in)
   (let* ((*print-level* (default-print-level *trace-print-level*))
          (*print-length* (default-print-length *trace-print-length*))
+         (*print-string-length* (default-print-string-length *trace-print-string-length*))
          (*print-readably* nil))
     (format *trace-output* "Calling ~S ~%" args)
     (force-output *trace-output*)))
@@ -95,6 +96,7 @@
   (declare (dynamic-extent args))
   (let* ((*print-level* (default-print-level *trace-print-level*))
          (*print-length* (default-print-length *trace-print-length*))
+         (*print-string-length* (default-print-string-length *trace-print-string-length*))
          (*print-readably* nil))
     (if (eq n 1)
       (progn
