@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (in-package "CCL")
 
@@ -755,8 +754,10 @@ are running on, or NIL if we can't find any useful information."
 	      (fresh-line *query-io*)
 	      (let* ((string (get-string-from-user "Selection [number,q,r,?]:"))
 		     (value (ignore-errors
-			      (let* ((*package* *keyword-package*))
-				(read-from-string string nil)))))
+                              (with-standard-io-syntax
+                                (let ((*read-eval* nil)
+                                      (*package* *keyword-package*))
+                                  (read-from-string string nil))))))
 		(cond ((eq value :q) (throw :cancel t))
 		      ((eq value :r) (throw :redisplay t))
 		      ((eq value :?) 
@@ -928,10 +929,20 @@ are running on, or NIL if we can't find any useful information."
         (native-translated-namestring
          (merge-pathnames "svnversion" *svn-program*)))
       "svnversion"))
-        
-                      
-        
-                         
+
+(defun local-git-revision ()
+  (let ((s (make-string-output-stream))
+	(git-dir (native-translated-namestring
+		  (merge-pathnames (ccl-directory) ".git"))))
+    (multiple-value-bind (status exit-code)
+	(external-process-status
+	 (run-program "git" (list "--git-dir" git-dir "describe" "HEAD")
+		      :output s :error :output))
+      (when (and (eq status :exited)
+		 (= exit-code 0))
+	(string-right-trim (list #\space #\newline)
+			   (get-output-stream-string s))))))
+
 (defun local-svn-revision ()
   (let* ((s (make-string-output-stream))
          (root (native-translated-namestring "ccl:")))
@@ -950,6 +961,9 @@ are running on, or NIL if we can't find any useful information."
               (return-from local-svn-revision line))))))
     nil))
 
+(defun local-vc-revision ()
+  (or (local-git-revision)
+      (local-svn-revision)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -1650,3 +1664,36 @@ are running on, or NIL if we can't find any useful information."
                                     +null-ptr+)))
         (unless (eql handle #$INVALID_HANDLE_VALUE)
           handle)))))
+;;; call into the CCL kernel to execute a CPUID instruction,  Yes it's
+;;; a PITA to need to do this
+#+x86-target
+(defun x86-cpuid (selector)
+  (rletz ((bx :unsigned-long)
+          (cx :unsigned-long )
+          (dx :unsigned-long))
+    (values
+     (external-call "cpuid"
+                    :unsigned-long selector
+                    :address bx
+                    :address cx
+                    :address dx
+                    :unsigned-long)
+     (pref bx :unsigned-long)
+     (pref cx :unsigned-long)
+     (pref dx :unsigned-long))))
+
+(defun winning-function-p (f)
+  (let* ((bits (lfun-bits f)))
+    (unless (logbitp $lfbits-noname-bit bits)
+      (logbitp 32 bits))))
+
+#+x86-target
+(defun rdtscp-p ()
+  (logbitp 27 (nth-value 3 (x86-cpuid #x80000001))))
+
+#+x86-target
+(defun invariant-tsc-p ()
+  (logbitp 8  (nth-value 3 (x86-cpuid #x80000007))))
+
+(defun get-errno ()
+  (- (%get-errno)))
