@@ -1,26 +1,27 @@
 /*
-   Copyright (C) 2010 Clozure Associates
-   Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Clozure CL.  
-
-   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-   License , known as the LLGPL and distributed with Clozure CL as the
-   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-   which is distributed with Clozure CL as the file "LGPL".  Where these
-   conflict, the preamble takes precedence.  
-
-   Clozure CL is referenced in the preamble as the "LIBRARY."
-
-   The LLGPL is also available online at
-   http://opensource.franz.com/preamble.html
-*/
+ * Copyright 1994-2010 Clozure Associates
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ * http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #define WORD_SIZE 32
 #define PLATFORM_OS PLATFORM_OS_LINUX
 #define PLATFORM_CPU PLATFORM_CPU_ARM
 #define PLATFORM_WORD_SIZE PLATFORM_WORD_SIZE_32
 
-typedef struct ucontext ExceptionInformation;
+#include <ucontext.h>
+
+typedef ucontext_t ExceptionInformation;
 
 #define MAXIMUM_MAPPABLE_MEMORY (3<<29)
 #define IMAGE_BASE_ADDRESS 0x10000000
