@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2009 Clozure Associates
-   Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Clozure CL.  
-
-   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-   License , known as the LLGPL and distributed+ with Clozure CL as the
-   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-   which is distributed with Clozure CL as the file "LGPL".  Where these
-   conflict, the preamble takes precedence.  
-
-   Clozure CL is referenced in the preamble as the "LIBRARY."
-
-   The LLGPL is also available online at
-   http://opensource.franz.com/preamble.html
-*/
+ * Copyright 1994-2009 Clozure Associates
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ * http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "lisp.h"
 #include "lisp_globals.h"
@@ -100,6 +99,11 @@ Boolean use_mach_exception_handling =
 #include <libgen.h>
 #endif
 
+#ifdef FREEBSD
+#include <sys/types.h>
+#include <sys/sysctl.h>
+#endif
+
 #if defined(FREEBSD) || defined(SOLARIS)
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -147,6 +151,8 @@ wperror(char* message)
 
 LispObj lisp_nil = (LispObj) 0;
 bitvector global_mark_ref_bits = NULL, dynamic_mark_ref_bits = NULL, relocatable_mark_ref_bits = NULL, global_refidx = NULL, dynamic_refidx = NULL,managed_static_refidx = NULL;
+LispObj *global_reloctab = (LispObj*) 0, *GCrelocptr = (LispObj*) 0;
+void *tcr_area_lock = NULL;
 
 
 /* These are all "persistent" : they're initialized when
@@ -1079,6 +1085,15 @@ determine_executable_name(char *argv0)
   return argv0;
 #endif
 #ifdef FREEBSD
+  int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
+  char exepath[PATH_MAX];
+  size_t len = sizeof(exepath);
+
+  if (sysctl(mib, 4, exepath, &len, NULL, 0) == 0) {
+    char *p = strndup(exepath, len);
+
+    return p;
+  }
   return ensure_real_path(argv0);
 #endif
 #ifdef SOLARIS
@@ -1445,7 +1460,7 @@ terminate_lisp()
 #endif
 
 #ifdef DARWIN
-#define min_os_version "8.0"    /* aka Tiger */
+#define min_os_version "13.0"    /* Mavericks */
 #endif
 #ifdef LINUX
 #ifdef PPC
@@ -1629,11 +1644,21 @@ check_x86_cpu()
   eax = cpuid(0, &ebx, &ecx, &edx);
 
   if (eax >= 1) {
+    int family;
+    int has_long_nop;
+
     eax = cpuid(1, &ebx, &ecx, &edx);
     cache_block_size = (ebx & 0xff00) >> 5;
-    if ((X86_REQUIRED_FEATURES & edx) == X86_REQUIRED_FEATURES) {
+
+    /* Does processor support multi-byte NOP (0x0f 0x1f)? */
+    family = (eax & 0xf00) >> 8;
+    has_long_nop = (family == 0x6 || family == 0xf);
+
+    if ((X86_REQUIRED_FEATURES & edx) == X86_REQUIRED_FEATURES && has_long_nop)
+    {
       return true;
     }
+
     /* It's very unlikely that SSE2 would be present and other things
        that we want wouldn't.  If they don't have MMX or CMOV either,
        might as well tell them. */
@@ -1646,7 +1671,9 @@ check_x86_cpu()
     if ((edx & X86_FEATURE_CMOV) == 0) {
       fprintf(dbgout, "This CPU doesn't support the CMOV instruction\n");
     }
-    
+    if (has_long_nop == 0) {
+      fprintf(dbgout, "This CPU doesn't support the multi-byte form of NOP\n");
+    }
   }
   return false;
 }
@@ -1665,33 +1692,13 @@ check_arm_cpu()
 
   install_signal_handler(SIGILL, (void *)early_signal_handler,0);
   feature_check();
-  win = arm_architecture_version >= 6;
+  win = arm_architecture_version >= 7;
   install_signal_handler(SIGILL, NULL, 0);
 
 #endif
   return win;
 }
-#endif  
-
-void
-lazarus()
-{
-  TCR *tcr = get_tcr(false);
-  if (tcr) {
-    /* Some threads may be dying; no threads should be created. */
-    LOCK(lisp_global(TCR_AREA_LOCK),tcr);
-    tcr->vs_area->active = tcr->vs_area->high - node_size;
-    tcr->save_vsp = (LispObj *)(tcr->vs_area->active);
-#ifndef ARM
-    tcr->ts_area->active = tcr->ts_area->high;
-    tcr->save_tsp = (LispObj *)(tcr->ts_area->active);
 #endif
-    tcr->catch_top = 0;
-    tcr->db_link = 0;
-    tcr->xframe = 0;
-    start_lisp(tcr, 0);
-  }
-}
 
 #ifdef LINUX
 #ifdef X8664
@@ -1703,6 +1710,12 @@ ensure_gs_available(char *progname)
 {
   LispObj fs_addr = 0L, gs_addr = 0L, cur_thread = (LispObj)pthread_self();
   char *gnu_get_libc_version(void);
+  /*
+   * According arch_prctl(2), there's no function prototype for
+   * arch_prctl().  Thus, we have to declare it ourselves.
+   * Note that addr is unsigned long * for GET operations.
+   */
+  extern int arch_prctl(int code, unsigned long *addr);
   
   arch_prctl(ARCH_GET_GS, &gs_addr);
   arch_prctl(ARCH_GET_FS, &fs_addr);
@@ -1904,7 +1917,7 @@ main
   }
 #endif
 #endif
-#ifdef WINDOXS
+#ifdef WINDOWS
   real_executable_name = determine_executable_name();
 #else
   real_executable_name = determine_executable_name(argv[0]);
@@ -2154,18 +2167,6 @@ main
   lisp_global(MANAGED_STATIC_REFBITS) = (LispObj)managed_static_refbits;
   lisp_global(MANAGED_STATIC_REFIDX) = (LispObj)managed_static_refidx;
   lisp_global(MANAGED_STATIC_DNODES) = (LispObj)managed_static_area->ndnodes;
-#ifdef ANDROID
-  /* In some versions of Android, atexit() generates a runtime warning
-     about the dangers of using atexit() with shared libraries.
-     Android is what it is.  It's a steaming pile of what it is, in fact.
-  */
-  {
-    extern int __cxa_atexit(void (*) (void *), void *, void *);
-    __cxa_atexit(lazarus, NULL, NULL);
-  }
-#else
-  atexit(lazarus);
-#endif
 #ifdef ARM
 #ifdef LINUX
 #ifdef SET_INITIAL_THREAD_AFFINITY
